@@ -10,6 +10,7 @@ from cryptography.hazmat.primitives.serialization import (
     load_pem_public_key,
 )
 
+from .key_interface import KeyInterface
 from .versions.v1 import V1Local, V1Public
 from .versions.v2 import V2Local, V2Public
 from .versions.v3 import V3Local, V3Public
@@ -17,9 +18,30 @@ from .versions.v4 import V4Local, V4Public
 
 
 class Key:
-    @staticmethod
-    def new(version: Union[int, str], type: str, key: Union[bytes, str] = b""):
+    """
+    Tha factory methods for PASETO keys.
+    """
 
+    @classmethod
+    def new(
+        cls, version: Union[int, str], type: str, key: Union[bytes, str] = b""
+    ) -> KeyInterface:
+
+        """
+        Constructor of a PASETO key object which has
+        :class:`KeyInterface <pyseto.key_interface.KeyInterface>`.
+
+        Args:
+            version(Union[int, str]): The version of the key. It will be ``1``,
+                ``2``, ``3`` or ``4``. ``str`` type of version (e.g., ``"v1"``)
+                can also be used but it will be DEPRECATED on ``v1.0.0``.
+            type (str): The type (purpose) of the key.
+            key (Union[bytes, str]): A key itself or keying material.
+        Returns:
+            KeyInterface: A PASETO key object.
+        Raise:
+            ValueError: Invalid arguments.
+        """
         if isinstance(version, str):
             if version == "v1":
                 version = 1
@@ -34,15 +56,7 @@ class Key:
 
         bkey = key if isinstance(key, bytes) else key.encode("utf-8")
         if type == "local":
-            if version == 1:
-                return V1Local(bkey)
-            if version == 2:
-                return V2Local(bkey)
-            if version == 3:
-                return V3Local(bkey)
-            if version == 4:
-                return V4Local(bkey)
-            raise ValueError(f"Invalid version: {version}.")
+            return cls._create_private_key(version, bkey)
 
         elif type == "public":
             k: Any = None
@@ -56,23 +70,79 @@ class Key:
                 k = load_pem_private_key(bkey, password=None)
             else:
                 raise ValueError("Invalid or unsupported PEM format.")
-            if version == 1:
-                return V1Public(k)
-            if version == 2:
-                return V2Public(k)
-            if version == 3:
-                return V3Public(k)
-            if version == 4:
-                return V4Public(k)
-            raise ValueError(f"Invalid version: {version}.")
+            return cls._create_public_key(version, k)
 
         raise ValueError(f"Invalid type(purpose): {type}.")
 
+    @staticmethod
+    def from_paserk(paserk: str) -> KeyInterface:
+
+        frags = paserk.split(".")
+        if frags[1] not in ["local", "public", "secret"]:
+            raise ValueError(f"Invalid PASERK key type: {frags[1]}.")
+
+        if frags[0] == "k1":
+            return (
+                V1Local.from_paserk(paserk)
+                if frags[1] == "local"
+                else V1Public.from_paserk(paserk)
+            )
+        if frags[0] == "k2":
+            return (
+                V2Local.from_paserk(paserk)
+                if frags[1] == "local"
+                else V2Public.from_paserk(paserk)
+            )
+        if frags[0] == "k3":
+            return (
+                V3Local.from_paserk(paserk)
+                if frags[1] == "local"
+                else V3Public.from_paserk(paserk)
+            )
+        if frags[0] == "k4":
+            return (
+                V4Local.from_paserk(paserk)
+                if frags[1] == "local"
+                else V4Public.from_paserk(paserk)
+            )
+        raise ValueError(f"Invalid PASERK version: {frags[0]}.")
+
+    # @classmethod
+    # def from_public_bytes(cls, version: int, key: bytes) -> KeyInterface:
+    #     if version == 1:
+    #         raise ValueError(f"RSA key is not supported.")
+    #     if version == 2:
+    #         return V2Public.from_public_bytes(key)
+    #     if version == 3:
+    #         return V3Public.from_public_bytes(key)
+    #     if version == 4:
+    #         return V4Public.from_public_bytes(key)
+    #     raise ValueError(f"Invalid version: {version}.")
+
     @staticmethod
     def from_asymmetric_key_params(
         version: Union[int, str], x: bytes = b"", y: bytes = b"", d: bytes = b""
-    ):
+    ) -> KeyInterface:
 
+        """
+        Constructor of a PASETO key object which has
+        :class:`KeyInterface <pyseto.key_interface.KeyInterface>` wth
+        asymmetric key parameters (x-coordinate, y-coordinate, and/or private
+        key). This is intended to be used to generate keys for PASETO from JWK
+        and other sources.
+
+        Args:
+            version(Union[int, str]): The version of the key. It will be ``1``,
+                ``2``, ``3`` or ``4``. ``str`` type of version (e.g., ``"v1"``)
+                can also be used but it will be DEPRECATED on ``v1.0.0``.
+            x (bytes): The x coordinate of the key.
+            y (bytes): The y coordinate of the key.
+            d (bytes): The private key component of the key.
+        Returns:
+            KeyInterface: A PASETO key object.
+        Raise:
+            ValueError: Invalid arguments.
+        """
         k: Any = None
 
         if isinstance(version, str):
@@ -148,3 +218,27 @@ class Key:
             raise ValueError("x or d should be set for v4.public.")
 
         raise ValueError(f"Invalid version: {version}.")
+
+    @staticmethod
+    def _create_public_key(version: int, key: Any) -> KeyInterface:
+        if version == 1:
+            return V1Public(key)
+        if version == 2:
+            return V2Public(key)
+        if version == 3:
+            return V3Public(key)
+        if version == 4:
+            return V4Public(key)
+        raise ValueError(f"Invalid version: {version}.")
+
+    @staticmethod
+    def _create_private_key(version: int, key: bytes) -> KeyInterface:
+        if version == 1:
+            return V1Local(key)
+        if version == 2:
+            return V2Local(key)
+        if version == 3:
+            return V3Local(key)
+        if version == 4:
+            return V4Local(key)
+        raise ValueError(f"Invalid version: {version}.")
