@@ -77,7 +77,7 @@ class KeyInterface:
         """
         Decrypts an encrypted PASETO token with the key.
 
-        This function is calld in :func:`pyseto.encode <pyseto.decode>`  so you
+        This function is calld in :func:`pyseto.decode <pyseto.decode>`  so you
         don't need to call it directly.
 
         Args:
@@ -126,7 +126,7 @@ class KeyInterface:
         """
         Verifies and decodes a signed PASETO token with the key.
 
-        This function is calld in :func:`pyseto.encode <pyseto.decode>`  so you
+        This function is calld in :func:`pyseto.decode <pyseto.decode>`  so you
         don't need to call it directly.
 
         Args:
@@ -143,3 +143,23 @@ class KeyInterface:
             NotSupportedError: The key does not support the operation.
         """
         raise NotSupportedError("A key for local does not have verify().")
+
+    def to_paserk(self) -> str:
+        """
+        Returns the PASERK expression of the key.
+
+        Returns:
+            str: A PASERK string.
+        """
+        raise NotImplementedError(
+            "The PASERK expression for the key is not supported yet."
+        )
+
+    def to_paserk_id(self) -> str:
+        """
+        Returns the PASERK ID of the key.
+
+        Returns:
+            str: A PASERK ID string.
+        """
+        raise NotImplementedError("The PASERK ID for the key is not supported yet.")
