@@ -1,9 +1,9 @@
 import hashlib
 import hmac
 from secrets import token_bytes
-from typing import Union
+from typing import Any, Union
 
-from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives import hashes, serialization
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric.ec import (
     EllipticCurvePrivateKey,
@@ -18,10 +18,11 @@ from cryptography.hazmat.primitives.kdf.hkdf import HKDF
 
 from ..exceptions import DecryptError, EncryptError, SignError, VerifyError
 from ..key_interface import KeyInterface
-from ..utils import base64url_encode, i2osp, os2ip, pae
+from ..local_key import LocalKey
+from ..utils import base64url_decode, base64url_encode, i2osp, os2ip, pae
 
 
-class V3Local(KeyInterface):
+class V3Local(LocalKey):
     """
     The key object for v3.local.
     """
@@ -31,6 +32,15 @@ class V3Local(KeyInterface):
         super().__init__(3, "local", key)
         return
 
+    @classmethod
+    def from_paserk(cls, paserk: str) -> KeyInterface:
+        frags = paserk.split(".")
+        if frags[0] != "k3":
+            raise ValueError("Invalid PASERK version for a v3.local key.")
+        if frags[1] != "local":
+            raise ValueError("Invalid PASERK type for a v3.local key.")
+        return cls(base64url_decode(frags[2]))
+
     def encrypt(
         self,
         payload: bytes,
@@ -112,13 +122,21 @@ class V3Local(KeyInterface):
         except Exception as err:
             raise DecryptError("Failed to decrypt a message.") from err
 
+    def to_paserk_id(self) -> str:
+        h = "k3.lid."
+        p = self.to_paserk()
+        digest = hashes.Hash(hashes.SHA384())
+        digest.update((h + p).encode("utf-8"))
+        d = digest.finalize()
+        return h + base64url_encode(d[0:33]).decode("utf-8")
+
 
 class V3Public(KeyInterface):
     """
     The key object for v3.public.
     """
 
-    def __init__(self, key: Union[str, bytes]):
+    def __init__(self, key: Any):
 
         super().__init__(3, "public", key)
         self._sig_size = 96
@@ -127,6 +145,29 @@ class V3Public(KeyInterface):
             raise ValueError("The key is not ECDSA key.")
         return
 
+    @classmethod
+    def from_paserk(cls, paserk: str) -> KeyInterface:
+        frags = paserk.split(".")
+        if frags[0] != "k3":
+            raise ValueError("Invalid PASERK version for a v3.public key.")
+        if frags[1] == "public":
+            return cls(
+                EllipticCurvePublicKey.from_encoded_point(
+                    ec.SECP384R1(), base64url_decode(frags[2])
+                )
+            )
+        # elif frags[1] == "secret":
+        #     return cls(Ed25519PrivateKey.from_private_bytes(base64url_decode(frags[2])))
+        raise ValueError("Invalid PASERK type for a v3.public key.")
+
+    @classmethod
+    def from_public_bytes(cls, key: bytes):
+        try:
+            k = EllipticCurvePublicKey.from_encoded_point(ec.SECP384R1(), key)
+        except Exception as err:
+            raise ValueError("Invalid bytes for the key.") from err
+        return cls(k)
+
     def sign(
         self, payload: bytes, footer: bytes = b"", implicit_assertion: bytes = b""
     ) -> bytes:
@@ -167,9 +208,34 @@ class V3Public(KeyInterface):
             raise VerifyError("Failed to verify.") from err
         return m
 
+    def to_paserk(self) -> str:
+        if isinstance(self._key, EllipticCurvePublicKey):
+            data = self._public_key_compress(
+                self._key.public_numbers().x, self._key.public_numbers().y
+            )
+            return "k3.public." + base64url_encode(data).decode("utf-8")
+        return (
+            "k3.secret."
+            + base64url_encode(
+                self._key.private_bytes(
+                    encoding=serialization.Encoding.DER,
+                    format=serialization.PrivateFormat.TraditionalOpenSSL,
+                    encryption_algorithm=serialization.NoEncryption(),
+                )
+            ).decode("utf-8")
+        )
+
+    def to_paserk_id(self) -> str:
+        p = self.to_paserk()
+        h = "k3.pid." if isinstance(self._key, EllipticCurvePublicKey) else "k3.sid."
+        digest = hashes.Hash(hashes.SHA384())
+        digest.update((h + p).encode("utf-8"))
+        d = digest.finalize()
+        return h + base64url_encode(d[0:33]).decode("utf-8")
+
     def _public_key_compress(self, x: int, y: int) -> bytes:
 
-        bx = x.to_bytes((x.bit_length() + 7) // 8, byteorder="big")
+        bx = x.to_bytes(48, byteorder="big")
         by = y.to_bytes((y.bit_length() + 7) // 8, byteorder="big")
         s = bytearray(1)
         s[0] = 0x02 + (by[len(by) - 1] & 1)
