@@ -4,6 +4,7 @@ import pytest
 
 import pyseto
 from pyseto import DecryptError, EncryptError, Key, VerifyError
+from pyseto.versions.v2 import V2Local, V2Public
 
 from .utils import load_key
 
@@ -63,6 +64,22 @@ class TestV2Local:
             pytest.fail("pyseto.encode() should fail.")
         assert "nonce must be 24 bytes long." in str(err.value)
 
+    @pytest.mark.parametrize(
+        "paserk, msg",
+        [
+            ("xx.local.AAAAAAAAAAAAAAAA", "Invalid PASERK version for a v2.local key."),
+            ("k3.local.AAAAAAAAAAAAAAAA", "Invalid PASERK version for a v2.local key."),
+            ("k2.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK type for a v2.local key."),
+            ("k2.public.AAAAAAAAAAAAAAAA", "Invalid PASERK type for a v2.local key."),
+        ],
+    )
+    def test_v2_local_from_paserk_with_invalid_args(self, paserk, msg):
+
+        with pytest.raises(ValueError) as err:
+            V2Local.from_paserk(paserk)
+            pytest.fail("Key.from_paserk should fail.")
+        assert msg in str(err.value)
+
 
 class TestV2Public:
     """
@@ -77,3 +94,25 @@ class TestV2Public:
             pyseto.decode(pk, token)
             pytest.fail("pyseto.decode() should fail.")
         assert "Failed to verify." in str(err.value)
+
+    @pytest.mark.parametrize(
+        "paserk, msg",
+        [
+            (
+                "xx.public.AAAAAAAAAAAAAAAA",
+                "Invalid PASERK version for a v2.public key.",
+            ),
+            (
+                "k3.public.AAAAAAAAAAAAAAAA",
+                "Invalid PASERK version for a v2.public key.",
+            ),
+            ("k2.xxx.AAAAAAAAAAAAAAAA", "Invalid PASERK type for a v2.public key."),
+            ("k2.local.AAAAAAAAAAAAAAAA", "Invalid PASERK type for a v2.public key."),
+        ],
+    )
+    def test_v2_public_from_paserk_with_invalid_args(self, paserk, msg):
+
+        with pytest.raises(ValueError) as err:
+            V2Public.from_paserk(paserk)
+            pytest.fail("Key.from_paserk should fail.")
+        assert msg in str(err.value)
